"""
Objects representing various MediaWiki pages.

This module extends pywikibot.page.
"""
from __future__ import annotations

import re
from contextlib import suppress
from functools import lru_cache
from typing import Any, Generator, Iterable, TypeVar, Union

import pywikibot
from pywikibot.textlib import removeDisabledParts


FilePageType = TypeVar("FilePageType", bound="FilePage")
PageSource = Union[
    pywikibot.page.Page, pywikibot.site.BaseSite, pywikibot.page.BaseLink
]
PageType = TypeVar("PageType", bound="Page")


@lru_cache()
def get_redirects(
    pages: frozenset[pywikibot.Page],
) -> frozenset[pywikibot.Page]:
    """Return all possible titles as pages for a set of pages."""
    link_pages = set()
    for page in pages:
        with suppress(pywikibot.exceptions.CircularRedirectError):
            while page.isRedirectPage():
                page = page.getRedirectTarget()
        if not page.exists():
            continue
        link_pages.add(page)
        link_pages.update(page.backlinks(filter_redirects=True))
    return frozenset(link_pages)


class Page(pywikibot.Page):
    """Represents a MediaWiki page."""

    BOT_START_END = re.compile(
        r"^(.*?<!--\s*bot start\s*-->).*?(<!--\s*bot end\s*-->.*)$",
        flags=re.I | re.S,
    )

    @classmethod
    def from_wikilink(
        cls: type[PageType],
        wikilink: object,
        site: pywikibot.site.BaseSite,
        default_namespace: int = 0,
    ) -> PageType:
        """
        Create a Page from a wikilink.

        :param wikilink: the wikilink text
        :param site: Site with the wikilink
        :param default_namespace: a namespace to use if the link does not
            contain one (defaults to 0)
        """
        text = removeDisabledParts(str(wikilink), site=site)
        text = text.strip().lstrip("[").rstrip("]")
        link = pywikibot.Link(text, site, default_namespace)
        return cls(link)

    def has_template(
        self, templates: str | Iterable[pywikibot.Page | str]
    ) -> bool:
        """
        Return True if the page has one of the templates. False otherwise.

        :param templates: templates to check
        """
        if isinstance(templates, str):
            templates = [templates]
        all_template_pages = get_redirects(
            frozenset(
                tpl
                if isinstance(tpl, pywikibot.Page)
                else Page(self.site, tpl, ns=10)
                for tpl in templates
            )
        )
        return bool(all_template_pages & set(self.templates()))

    @property
    def is_article(self) -> bool:
        """Return True if the page is an article. False otherwise."""
        return (
            self.namespace() == 0
            and not self.isDisambig()
            and not self.isRedirectPage()
        )

    def save_bot_start_end(
        self,
        text: str,
        *,
        minor: bool | None = False,
        botflag: bool | None = False,
        **kwargs: Any,
    ) -> None:
        """
        Save text to the specified area of the page.

        The area is specified by <!--bot start--> and <!--bot end-->.

        :param text: text to save to the page
        :param minor: if True, mark this edit as minor
        :param botflag: if True, mark this edit as made by a bot

        See :meth:`pywikibot.page.Page.save` for other arguments.
        """
        if not self.exists():
            pywikibot.warning(f"{self.title()} does not exist. Skipping.")
            return
        text = text.strip()
        current_text = self.text  # type: ignore[has-type]
        if self.BOT_START_END.match(current_text):
            self.text = self.BOT_START_END.sub(fr"\1\n{text}\2", current_text)
        else:
            self.text = text
        self.save(minor=minor, botflag=botflag, **kwargs)


class FilePage(pywikibot.FilePage, Page):
    """Represents a file description page."""

    @classmethod
    def from_wikilink(
        cls: type[FilePageType],
        wikilink: object,
        site: pywikibot.site.BaseSite,
        default_namespace: int = 6,
    ) -> FilePageType:
        """
        Create a FilePage from a wikilink.

        :param wikilink: the wikilink text
        :param site: Site with the wikilink
        :param default_namespace: a namespace to use if the link does not
            contain one (defaults to 6)
        """
        return super().from_wikilink(wikilink, site, default_namespace)

    @property
    def is_used(self) -> bool:
        """Return True if the file is used, False otherwise."""
        return len(set(self.usingPages())) > 0

    @property
    def megapixels(self) -> float | None:
        """
        Return the file's megapixels.

        Returns None if the dimensions are 0 or unknown.
        """
        try:
            height = self.latest_file_info.height
            width = self.latest_file_info.width
        except AttributeError:
            return None
        else:
            return height * width / 1e6 or None

    def usingPages(  # noqa: N802
        self, total: int | None = None, content: bool = False
    ) -> Generator[Page, None, None]:
        """Yield pages on which the file is displayed."""
        for page in super().usingPages(total=total, content=content):
            # MediaWiki considers file redirects to be using the target flie,
            # but they are not actually using it.
            if not (page.namespace() == 6 and page.isRedirectPage()):
                yield Page(page)



"""
Functions for manipulating wikitext.

This module extends pywikibot.textlib.
"""
from typing import Iterable

from pywikibot.page import BasePage


# The namespace names must be substituted into this regex,
# e.g., FILE_LINK_REGEX.format("|".join(site.namespaces.FILE)).
# Adds named groups to pywikibot.textlib.FILE_LINK_REGEX
# and uses str.format instead of %.
FILE_LINK_REGEX = r"""
    \[\[\s*
    (?P<namespace>{})
    \s*:
    (?=(?P<filename>
        [^]|]*
    ))(?P=filename)
    (
        \|
        (
            (
                (?=(?P<inner_link>
                    \[\[.*?\]\]
                ))(?P=inner_link)
            )?
            (?=(?P<other_chars>
                [^\[\]]*
            ))(?P=other_chars)
        |
            (?=(?P<not_wikilink>
                \[[^]]*\]
            ))(?P=not_wikilink)
        )*?
    )??
    \]\]
"""


def iterable_to_wikitext(
    items: Iterable[object], *, prefix: str = "\n* "
) -> str:
    """
    Convert iterable to wikitext.

    Pages are converted to links.
    All other objects use their string representation.

    :param items: Items to iterate
    :param prefix: Prefix for each item when there is more than one item
    """
    if not items:
        return ""
    if len(list(items)) == 1:
        prefix = ""
    text = ""
    for item in items:
        if isinstance(item, BasePage):
            item = item.title(as_link=True, textlink=True)
        text += f"{prefix}{item}"
    return text



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pywikibot_extensions']

package_data = \
{'': ['*']}

install_requires = \
['pywikibot>=6.5.0,<7.0.0']

setup_kwargs = {
    'name': 'pywikibot-extensions',
    'version': '21.9.1',
    'description': 'Extends the Pywikibot library',
    'long_description': '[![Latest PyPI release](https://img.shields.io/pypi/v/pywikibot-extensions?logo=pypi)](https://pypi.org/project/pywikibot-extensions) ![Latest GitHub release](https://img.shields.io/github/v/release/JJMC89/pywikibot-extensions?logo=github) ![Latest tag](https://img.shields.io/github/v/tag/JJMC89/pywikibot-extensions?logo=git)\n\n![License](https://img.shields.io/pypi/l/pywikibot-extensions?color=blue) ![Python versions](https://img.shields.io/pypi/pyversions/pywikibot-extensions?logo=python)\n\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/JJMC89/pywikibot-extensions/main.svg)](https://results.pre-commit.ci/latest/github/JJMC89/pywikibot-extensions/main) [![CI](https://github.com/JJMC89/pywikibot-extensions/actions/workflows/ci.yaml/badge.svg?branch=main)](https://github.com/JJMC89/pywikibot-extensions/actions?query=workflow%3ACI+branch%3Amain)\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n# pywikibot-extensions\nThe pywikibot-extensions library extends the [Pywikibot](https://pywikibot.org) library.\n\n## Installation\n`pip install pywikibot-extensions`\n',
    'author': 'JJMC89',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/JJMC89/pywikibot-extensions',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
